#include "vesavbe.h"
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>


 /*-----------------01-19-98 02:25pm-----------------
  * Small test program which shows several VESA informations,
  * a modelist and a couple of warnings
  * --------------------------------------------------*/

static VBE_VbeInfoBlock     vbei;
static VBE_ModeInfoBlock    ModeInfo;

void cleartextscreen (void);
// just clears the text-screen
#pragma aux cleartextscreen = "pushad            " \
			      "mov edi, 0b8000h  " \
    			      "mov eax, 7200720h " \
    			      "mov ecx, 1000     " \
    			      "rep stosd         " \
    			      "mov ah, 02h       " \
    			      "xor ebx, ebx      " \
    			      "xor edx, edx      " \
    			      "int 10h           " \
    			      "popad             " \
			      modify[]

void ControllerTest(void)
{
  int modes;  

  cleartextscreen();
  for (modes=0; vbei.VideoModePtr[modes]!=0xffff; modes++);
  printf ("\n\n");
  printf ("  VESA VBE 2.0 Controller Information Block\n");
  printf ("  -----------------------------------------\n\n");
  printf ("  Signature                %4s\n",   vbei.vbeSignature);
  printf ("  Version                  %hi.%hi\n", vbei.vbeVersion.hi, vbei.vbeVersion.lo);
  printf ("  OEM String               %s\n",    vbei.OemStringPtr);
  printf ("  Total Memory             %hi kb\n",   vbei.TotalMemory*64);
  printf ("  Oem Software Rev:        %hi\n",   vbei.OemSoftwareRev);
  printf ("  Oem Vendor Name          %s\n",    vbei.OemVendorNamePtr);
  printf ("  Oem Product Name         %s\n",    vbei.OemProductNamePtr);
  printf ("  Oem Product Rev          %s\n",    vbei.OemProductRevPtr);
  printf ("  Number of Video-Modes    %i\n",    modes);
  printf ("\n");
  printf ("  Capabilities:\n");
  printf ("  -------------\n");
  printf ("  8-Bit DAC support        %s\n", (vbei.Capabilities & CAP_8bit_DAC) ?              "Yes": "No");
  printf ("  VGA-Compatibility        %s\n", (vbei.Capabilities & CAP_VGACompatible) ?         "Yes": "No");
  printf ("  should use DAC functions %s\n", (vbei.Capabilities & CAP_Use_VBE_DAC_Functions) ? "Yes": "No");
  getch();
}

void Warnings (void)
{
  cleartextscreen();
  int falsebitdepth = 0;
  for ( int mode=0; vbei.VideoModePtr[mode]!=0xffff; mode++)
  {
    VBE_Mode_Information (vbei.VideoModePtr[mode], &ModeInfo);
    // look for 16 bit modes which are only 15 bits wide
    if  ( (ModeInfo.BitsPerPixel == 16) &&
	  ((ModeInfo.RedMaskSize +
	   ModeInfo.GreenMaskSize +
	   ModeInfo.BlueMaskSize) != 16)) falsebitdepth = 1;
  }

  printf ("\n\n");
  printf ("  VESA VBE 2.0 Warnings\n");
  printf ("  ---------------------\n\n");
  printf ("  Program should not access any VGA-Ports directly %s\n", (vbei.Capabilities & CAP_VGACompatible) ?         "No": "Yes");
  printf ("  Program should not access the DAC directly       %s\n", (vbei.Capabilities & CAP_Use_VBE_DAC_Functions) ? "Yes": "No");
  printf ("  Bios treats 15 bit modes as 16 bit               %s\n", falsebitdepth ? "Yes": "No");
  getch();
}
char *Types[8] = {"Text","CGA","Hercules","Planar","Packed Pixel","Unchained","Directcolor","YUV"};

void ModeListTest (void)
{
  int i, scroll, mode;
  cleartextscreen();
  scroll=0;
  printf ("Mode-Number  XRes   YRes  BPP  RGB   Type         Pages\n");
  printf ("------------------------------------------------------------------------------\n");
  for ( mode=0; vbei.VideoModePtr[mode]!=0xffff; mode++)
  {
    VBE_Mode_Information (vbei.VideoModePtr[mode], &ModeInfo);
    printf ("0x%3x        %4i   %4i  %2i   %hi%hi%hi   %-12s %2hi\n",
       vbei.VideoModePtr[mode],
       ModeInfo.XResolution,
       ModeInfo.YResolution,
       ModeInfo.BitsPerPixel,
       ModeInfo.RedMaskSize,
       ModeInfo.GreenMaskSize,
       ModeInfo.BlueMaskSize,
       Types[ModeInfo.MemoryModel],
       ModeInfo.NumberOfImagePages);

    if (scroll++ == 20)
    {
      getch();
      scroll=0;
      cleartextscreen();
      printf ("Mode-Number  XRes   YRes  BPP  RGB   Type         Pages\n");
      printf ("------------------------------------------------------------------------------\n");
    }
  }
  if (scroll) getch();
}

void mainmenu (void)
{
  char ch = 0;
  do
  {
    cleartextscreen ();
    printf ("\n\n");
    printf ("  VESA VBE 2.0 Test Utility\n");
    printf ("  -------------------------\n\n");
    printf ("  1)   VESA VBE Controller Information\n");
    printf ("  2)   VESA VBE 2.0 Mode-List\n");
    printf ("  3)   Warnings\n");
    printf ("\n");
    printf ("  ESC  Exit Test-Utility\n");
    ch=getch();
    switch ( ch )
    {
      case '1': ControllerTest(); break;
      case '2': ModeListTest();   break;
      case '3': Warnings ();      break;
    }
  } while (ch!=27);
}

void main (void)
{
  // VESA VBE 2.0 Interface Information
  VBE_Init();
  VBE_Controller_Information (&vbei);
  mainmenu ();
  VBE_Done();
}


